(***********************************************************************)
(*                                                                     *)
(*                        Call/cc for ocamlc                           *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 2005 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file LICENSE.        *)
(*                                                                     *)
(***********************************************************************)

(* $Id: test.ml,v 1.2 2005/10/05 08:16:49 xleroy Exp $ *)

open Callcc

let f a =
  callcc (fun k -> 1 + (if a then 2 else throw k 10))

let coroutines () =
  callcc (fun init_k ->
    let curr_k = ref init_k in
    let communicate x = callcc (fun k -> let old_k = !curr_k in curr_k := k; throw old_k x) in
    let rec process1 n =
      if n >= 30 then begin
        print_string "1: finished"; print_newline(); 100
      end else begin
        print_string "1: received "; print_int n; print_newline();
        process1(communicate(n+1))
      end
    and process2 n =
      if n >= 20 then begin
        print_string "2: finished"; print_newline(); 15
      end else begin
        print_string "2: received "; print_int n; print_newline();
        process2(communicate(n+1))
      end in
    process1(callcc(fun start1 ->
      process2(callcc(fun start2 ->
        curr_k := start2; throw start1 0)))))

exception E1

let exns a =
  try
    try
      try
        if callcc (fun k -> 1 + (if a then 2 else throw k 10)) = 10
        then raise E1
        else begin print_string "No exception"; print_newline() end
      with Failure s ->
        print_string "Failure "; print_string s; print_newline()
    with E1 ->
      print_string "Exception E1"; print_newline(); failwith "cascade"
  with Failure s ->
    print_string "Failure "; print_string s; print_newline()

let overapplication x1 x2 =
  callcc (fun k -> fun x3 x4 x5 -> throw k (fun y3 y4 y5 -> 42))

let _ =
  print_int (f true); print_newline();
  print_int (f false); print_newline();
  exns true;
  exns false;
  print_int (overapplication 1 2 3 4 5); print_newline();
  coroutines()

